# frozen_string_literal: true
module Wilco
  class Command
    def initialize(parser)
      @parser = parser
    end

    def likelihood(message)
      @likelihood = parser.command_likelihood(tokens, message)
    end

    def parse_message(message)
      @param_values = parser.extract_params(tokens, message)
      self
    end

    def missing_params?
      !first_missing_param.nil?
    end

    def request_param(new_param)
      @current_param = new_param
      switch_param_message
    end

    def request_first_missing_param
      @current_param = first_missing_param
      missing_param_message
    end

    def extract_param(message)
      param = params[current_param] # TODO: rename to param name
      new_param_values = parser.extract_params([param], message)
      param_values.merge!(new_param_values) if new_param_values[param[:name]]
    end

    def find_param_to_switch_to(message)
      parser.find_param_name(tokens, message)
    end

    def execute
      confirmation_message
    end

    private

    attr_reader :parser, :param_values, :current_param

    def first_missing_param
      param_values.keys.find { |key| param_values[key].nil? }
    end
  end
end
