# frozen_string_literal: true
require "pry"

module Wilco
  class Parser
    def command_likelihood(tokens, message)
      words = tokenize_message(message)
      static_tokens(tokens).count do |token|
        match_static_token_in_message(token, words)
      end
    end

    def extract_params(tokens, message)
      params = {}
      words = tokenize_message(message)
      param_tokens(tokens).each do |token|
        params[token[:name]] = extract_param_from_message(token, words)
      end
      params
    end

    def find_param_name(tokens, message)
      words = tokenize_message(message)
      token = param_tokens(tokens).find { |t|
        extract_param_name_from_message(t, words)
      }
      token[:name] if token
    end

    private

    attr_accessor :params

    def static_tokens(tokens)
      tokens.select { |t| t.is_a?(Array) }
    end

    def param_tokens(tokens)
      tokens.select { |t| t.is_a?(Hash) }
    end

    def tokenize_message(message)
      simplify_string(message.strip).split(/\s+/)
    end

    def simplify_string(string)
      string.downcase.gsub(/[^\w\s]+/, "")
    end

    def match_static_token_in_message(token, words)
      token.each do |token_option|
        return words if words.include?(token_option)
      end
      false
    end

    def extract_param_from_message(token, words)
      value = token[:values].find { |synonyms|
        words_include_a_synonym?(words, synonyms)
      }
      value[0] if value
    end

    def extract_param_name_from_message(token, words)
      words.include?(token[:name].to_s) # TODO: store as a string originally
    end

    def words_include_a_synonym?(words, synonyms)
      synonyms.any? { |synonym| words.include?(simplify_string(synonym)) }
    end

    def set_param(token, value)
      params[token[:name]] = value
    end
  end
end
